-------------------------------------------------------------------------------
-- $Id$
-------------------------------------------------------------------------------
-- FIFO.vhd
-------------------------------------------------------------------------------
--
--                  ****************************
--                  ** Copyright Xilinx, Inc. **
--                  ** All rights reserved.   **
--                  ****************************
--
-------------------------------------------------------------------------------
-- Filename:        FIFO.vhd
--
-- Description:     
--                  
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--              FIFO.vhd
--
-------------------------------------------------------------------------------
-- Author:          goran
-- Revision:        $Revision$
-- Date:            $Date$
--
-- History:
--   goran  2001-06-08    First Version
--
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity FIFO is
  
  generic (
    C_VIRTEXII : boolean := true;       -- True if target is VII, false if
                                        -- target is Virtex, VirtexE or SpartanII
    C_DEPTH    : natural := 10;         -- Nr of bits needed for the counters
    C_WIDTH    : natural := 16);        -- Nr of bits needed for the data

  port (
    Reset      : in std_logic;
    Reset_FIFO : in  std_logic;

    Write_Clk  : in  std_logic;
    Write      : in  std_logic;
    Write_Data : in  std_logic_vector(C_WIDTH-1 downto 0);
    Full       : out std_logic;

    Read_Clk   : in  std_logic;
    Read       : in  std_logic;
    Read_Data  : out std_logic_vector(C_WIDTH-1 downto 0);
    Empty      : out std_logic);

end entity FIFO;

architecture IMP of FIFO is

  component Grey_Counter is
    generic (
      C_SIZE : natural);
    port (
      Clk        : in  std_logic;
      Reset      : in  std_logic;
      Count      : in  std_logic;
      Bin_Value  : out std_logic_vector(C_SIZE-1 downto 0);
      Grey_Value : out std_logic_vector(C_SIZE-1 downto 0));
  end component Grey_Counter;

  component Flag_Handling is
    generic (
      C_SIZE : natural);
    port (
      Reset : in std_logic;

      Read_Clk          : in std_logic;
      Read_Counter_Grey : in std_logic_vector(C_SIZE-1 downto 0);

      Write_Clk          : in std_logic;
      Write_Counter_Grey : in std_logic_vector(C_SIZE-1 downto 0);

      Full  : out std_logic;
      Empty : out std_logic);
  end component Flag_Handling;

  component FIFO_Memory is
    generic (
      C_VIRTEXII : boolean;             -- True if target is VII, false if
      -- target is Virtex,VirtexE,SpartanII
      C_DEPTH    : natural;             -- Nr of bits needed for the counters
      C_WIDTH    : natural);            -- Nr of bits needed for the data
    port (
      Write_Clk  : in std_logic;
      Write      : in std_logic;
      Write_Addr : in std_logic_vector(C_DEPTH-1 downto 0);
      Write_Data : in std_logic_vector(C_WIDTH-1 downto 0);

      Read_Clk  : in  std_logic;
      Read_Addr : in  std_logic_vector(C_DEPTH-1 downto 0);
      Read_Data : out std_logic_vector(C_WIDTH-1 downto 0));
  end component FIFO_Memory;

  signal full_reset : std_logic;

  signal write_counter_bin  : std_logic_vector(C_DEPTH-1 downto 0);
  signal write_counter_grey : std_logic_vector(C_DEPTH-1 downto 0);

  signal read_counter_bin  : std_logic_vector(C_DEPTH-1 downto 0);
  signal read_counter_grey : std_logic_vector(C_DEPTH-1 downto 0);
  
begin  -- architecture IMP

  -- Oring the two reset signals into one
  full_reset <= Reset or Reset_FIFO;

  -----------------------------------------------------------------------------
  -- Implementing the grey counters
  -----------------------------------------------------------------------------
  Write_Grey_Counter_I : Grey_Counter
    generic map (
      C_SIZE => C_DEPTH)                -- [natural]
    port map (
      Clk        => Write_Clk,          -- [in  std_logic]
      Reset      => full_reset,         -- [in  std_logic]
      Count      => Write,              -- [in  std_logic]
      Bin_Value  => write_counter_bin,  -- [out std_logic_vector(Size-1 downto 0)]
      Grey_Value => write_counter_grey);  -- [out std_logic_vector(Size-1 downto 0)]

  Read_Grey_Counter_I : Grey_Counter
    generic map (
      C_SIZE => C_DEPTH)                -- [natural]
    port map (
      Clk        => Read_Clk,           -- [in  std_logic]
      Reset      => full_reset,         -- [in  std_logic]
      Count      => Read,               -- [in  std_logic]
      Bin_Value  => read_counter_bin,  -- [out std_logic_vector(Size-1 downto 0)]
      Grey_Value => read_counter_grey);  -- [out std_logic_vector(Size-1 downto 0)]

  -----------------------------------------------------------------------------
  -- Instanciating the flag handling
  -----------------------------------------------------------------------------
  Flag_Handling_I : Flag_Handling
    generic map (
      C_SIZE => C_DEPTH)                -- [natural]
    port map (
      Reset              => full_reset,         -- [in  std_logic]
      Read_Clk           => Read_Clk,   -- [in  std_logic]
      Read_Counter_Grey  => read_counter_grey,  -- [in  std_logic_vector(C_SIZE-1 downto 0)]
      Write_Clk          => Write_Clk,  -- [in  std_logic]
      Write_Counter_Grey => write_counter_grey,  -- [in  std_logic_vector(C_SIZE-1 downto 0)]
      Full               => Full,       -- [out std_logic]
      Empty              => Empty);     -- [out std_logic]

  -----------------------------------------------------------------------------
  -- Instanciating the FIFO Memory
  -----------------------------------------------------------------------------
  FIFO_Memory_I : FIFO_Memory
    generic map (
      C_VIRTEXII => C_VIRTEXII,  -- [boolean] True if target is VII, false if
      C_DEPTH    => C_DEPTH,  -- [natural] Nr of bits needed for the counters
      C_WIDTH    => C_WIDTH)  -- [natural] Nr of bits needed for the data
    port map (
      Write_Clk  => Write_Clk,          -- [in  std_logic]
      Write      => Write,              -- [in  std_logic]
      Write_Addr => write_counter_bin,  -- [in  std_logic_vector(C_DEPTH-1 downto 0)]
      Write_Data => Write_Data,  -- [in  std_logic_vector(C_WIDTH-1 downto 0)]
      Read_Clk   => Read_Clk,           -- [in  std_logic]
      Read_Addr  => read_counter_bin,  -- [in  std_logic_vector(C_DEPTH-1 downto 0)]
      Read_Data  => Read_Data);  -- [out std_logic_vector(C_WIDTH-1 downto 0)]
end architecture IMP;
