-------------------------------------------------------------------------------
-- $Id: zbt_controller_core.vhd,v 1.2 2001/12/20 00:51:04 goran Exp $
-------------------------------------------------------------------------------
-- zbt_controller_core.vhd
-------------------------------------------------------------------------------
--
--                  ****************************
--                  ** Copyright Xilinx, Inc. **
--                  ** All rights reserved.   **
--                  ****************************
--
-------------------------------------------------------------------------------
-- Filename:        zbt_controller_core.vhd
--
-- Description:     
--                  
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--              zbt_controller_core.vhd
--
-------------------------------------------------------------------------------
-- Author:          goran
-- Revision:        $Revision: 1.2 $
-- Date:            $Date: 2001/12/20 00:51:04 $
--
-- History:
--   goran  2001-06-12    First Version
--
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;

entity Zbt_controller_Core is
  generic (
    C_ZBT_ADDR_SIZE : integer := 17;
    -- If C_EXTERNAL_DLL is true (1) then
    -- ZBT_Clk is driven outside this module
    C_EXTERNAL_DLL  : integer := 0
    );
  port (
    Clk   : in std_logic;
    Reset : in std_logic;

    -- OPB signals
    ZBT_CS      : in std_logic;
    OPB_ABus    : in std_logic_vector(0 to C_ZBT_ADDR_SIZE-1);
    OPB_BE      : in std_logic_vector(0 to 3);
    OPB_RNW     : in std_logic;
    OPB_seqAddr : in std_logic;
    OPB_DBus    : in std_logic_vector(0 to 31);

    ZBT_DBus    : out std_logic_vector(0 to 31);
    ZBT_xferAck : out std_logic;

    Real_OPB_xferAck : in std_logic;

    -- Video signals
    Video_CS      : in  std_logic;
    Video_Adress  : in  std_logic_vector(0 to C_ZBT_ADDR_SIZE-1);
    Video_Data    : out std_logic_vector(0 to 31);
    Video_xferAck : out std_logic;

    -- ZBT Memory signals
    ZBT_Clk_FB    : in  std_logic;
    ZBT_Clk_FBOut : out std_logic;
    ZBT_Clk       : out std_logic;
    ZBT_OE_N      : out std_logic;
    ZBT_ADV_LD_N  : out std_logic;
    ZBT_LBO_N     : out std_logic;
    ZBT_CE1_N     : out std_logic;
    ZBT_CE2_N     : out std_logic;
    ZBT_CE2       : out std_logic;
    ZBT_RW_N      : out std_logic;
    ZBT_CKE_N     : out std_logic;
    ZBT_A         : out std_logic_vector(0 to C_ZBT_ADDR_SIZE-1);
    ZBT_BW_N      : out std_logic_vector(1 to 4);
    ZBT_IO_I      : in  std_logic_vector(0 to 31);
    ZBT_IO_O      : out std_logic_vector(0 to 31);
    ZBT_IO_T      : out std_logic;
    ZBT_IOP_I     : in  std_logic_vector(1 to 4);
    ZBT_IOP_O     : out std_logic_vector(1 to 4);
    ZBT_IOP_T     : out std_logic
    );

end entity Zbt_controller_Core;

library unisim;
use unisim.all;

architecture IMP of Zbt_controller_Core is

  component IBUFG is
    port (
      I : in  std_logic;
      O : out std_logic);
  end component IBUFG;

  component CLKDLL
    port (
      CLKIN  : in  std_logic := '0';
      CLKFB  : in  std_logic := '0';
      RST    : in  std_logic := '0';
      CLK0   : out std_logic := '0';
      CLK90  : out std_logic := '0';
      CLK180 : out std_logic := '0';
      CLK270 : out std_logic := '0';
      CLK2X  : out std_logic := '0';
      CLKDV  : out std_logic := '0';
      LOCKED : out std_logic := '0');
  end component;

  component FDRE is
    -- pragma translate_off
    generic (
      INIT : bit);
    -- pragma translate_on
    port (
      Q  : out std_logic;
      C  : in  std_logic;
      CE : in  std_logic;
      D  : in  std_logic;
      R  : in  std_logic);
  end component FDRE;

  component FDR is
    port (
      Q : out std_logic;
      C : in  std_logic;
      D : in  std_logic;
      R : in  std_logic);
  end component FDR;

  component FDE is
    port (
      Q  : out std_logic;
      C  : in  std_logic;
      CE : in  std_logic;
      D  : in  std_logic);
  end component FDE;

  signal zbt_Clk_FB_I : std_logic;

  signal CLK0   : std_logic;
  signal CLK90  : std_logic;
  signal CLK180 : std_logic;
  signal CLK270 : std_logic;
  signal CLK2X  : std_logic;
  signal CLKDV  : std_logic;
  signal LOCKED : std_logic;

  signal zbt_rw_I_n : std_logic;
  signal zbt_ce2_I  : std_logic;

  signal zbt_cs_i        : std_logic;
  signal opb_in_progress : std_logic;

  signal zbt_wrD_2   : std_logic_vector(OPB_DBus'range);
  signal zbt_rw_2_n  : std_logic;
  signal zbt_data_in : std_logic_vector(OPB_DBus'range);

  signal xfer_Ack_1       : std_logic;
  signal xfer_Ack_1_OPB   : std_logic;
  signal xfer_Ack_1_OPB_i : std_logic;
  signal xfer_Ack_2_OPB   : std_logic;
  signal xfer_Ack_3       : std_logic;
  signal xfer_Ack_3_OPB   : std_logic;
  signal xfer_Ack_4_OPB   : std_logic;
  signal xfer_Ack_OPB_i   : std_logic;
  signal xfer_Ack_OPB     : std_logic;
  signal xfer_Ack_1_Video : std_logic;
  signal xfer_Ack_2_Video : std_logic;
  signal xfer_Ack_3_Video : std_logic;
  signal xfer_Ack_4_Video : std_logic;

  signal clk_i : std_logic;
  
begin  -- architecture IMP

  -----------------------------------------------------------------------------
  -- Tie all unused control signals to correct values
  -----------------------------------------------------------------------------
  ZBT_ADV_LD_N <= '0';
  ZBT_CKE_N    <= '0';
  ZBT_LBO_N    <= '0';
  ZBT_CE1_N    <= '0';
  ZBT_CE2_N    <= '0';
  ZBT_OE_N     <= '0';


  -----------------------------------------------------------------------------
  -- Generating the ZBT_Clk
  -----------------------------------------------------------------------------

  Instanciate_DLL : if (C_EXTERNAL_DLL = 0) generate

    CLKDLL_I : CLKDLL
      port map (
        CLKIN  => Clk,                  -- [in  std_logic := '0']
        CLKFB  => zbt_Clk_FB,           -- [in  std_logic := '0']
        RST    => Reset,                -- [in  std_logic := '0']
        CLK0   => clk_i,                -- [out std_logic := '0']
        CLK90  => CLK90,                -- [out std_logic := '0']
        CLK180 => CLK180,               -- [out std_logic := '0']
        CLK270 => CLK270,               -- [out std_logic := '0']
        CLK2X  => CLK2X,                -- [out std_logic := '0']
        CLKDV  => CLKDV,                -- [out std_logic := '0']
        LOCKED => LOCKED);              -- [out std_logic := '0']

    ZBT_Clk       <= clk_i;
    ZBT_Clk_FBOut <= clk_i;
    
  end generate Instanciate_DLL;

  -----------------------------------------------------------------------------
  -- Driving the address and control signals
  -----------------------------------------------------------------------------

  zbt_cs_i <= zbt_cs and not Video_CS;

  opb_in_progress_DFF : FDR
    port map (
      Q => opb_in_progress,             -- [out std_logic]
      C => Clk,                         -- [in  std_logic]
      D => zbt_cs_i,                    -- [in  std_logic]
      R => Real_OPB_xferAck);           -- [in std_logic]

  -- First clock all signals, these will hopefully be place in IOB
  Addr_Ctrl_DFFs : process (Clk) is
  begin  -- process Addr_Ctrl_DFFs
    if Clk'event and Clk = '1' then     -- rising clock edge
      if (Video_CS = '1') then
        ZBT_A      <= Video_Adress;
        ZBT_BW_N   <= (others => '0');
        zbt_ce2_I  <= '1';
        zbt_rw_I_n <= '1';
      elsif (ZBT_CS = '1') and (opb_in_progress = '0') then
        ZBT_A      <= OPB_ABus;
        ZBT_BW_N   <= not OPB_BE;
        zbt_ce2_I  <= '1';
        zbt_rw_I_n <= OPB_RNW;
      else
        zbt_ce2_I  <= '0';
        zbt_rw_I_n <= '1';
      end if;
    end if;
  end process Addr_Ctrl_DFFs;

  ZBT_RW_N <= zbt_rw_I_n;
  ZBT_CE2  <= zbt_ce2_I;

  -----------------------------------------------------------------------------
  -- Handling the Data signals
  -----------------------------------------------------------------------------
  Write_Data_DFFs : process (Clk, Reset) is
  begin  -- process Write_Data_DFFs
    if Reset = '1' then                 -- asynchronous reset (active high)
      zbt_wrD_2  <= ( others => '0');
      zbt_rw_2_n <= '0';
    elsif Clk'event and Clk = '1' then  -- rising clock edge
      if (zbt_ce2_I = '1') then
        zbt_wrD_2 <= OPB_DBus;
      end if;
      ZBT_IO_O   <= zbt_wrD_2;
      zbt_rw_2_n <= not(not zbt_rw_I_n and xfer_Ack_1);  -- '0' means to drive Output (Write)
                                        -- and '1' means turn-off Output (Read)
      ZBT_IO_T   <= zbt_rw_2_n;
      ZBT_IOP_T  <= zbt_rw_2_n;
    end if;
  end process Write_Data_DFFs;

  ZBT_IOP_O <= (others => '1');

  ZBT_DBus_DFF : for I in ZBT_DBus'range generate
    ZBT_Bus_FDE : FDE
      port map (
        Q  => zbt_data_in(I),           -- [out std_logic]
        C  => Clk,                      -- [in  std_logic]
        CE => xfer_Ack_3,               -- [in  std_logic]
        D  => ZBT_IO_I(I));             -- [in  std_logic]
  end generate ZBT_DBus_DFF;

  Video_Data <= zbt_data_in;
  ZBT_DBus   <= zbt_data_in;

  -----------------------------------------------------------------------------
  -- Handling the xferAck signal
  -----------------------------------------------------------------------------
  xfer_Ack_1_OPB_i <= ZBT_CS and not Video_CS;

  xfer_ack_1_opb_dff : FDR
    port map (
      Q => xfer_Ack_1_OPB,              -- [out std_logic]
      C => Clk,                         -- [in  std_logic]
      D => xfer_Ack_1_OPB_i,            -- [in  std_logic]
      R => Real_OPB_xferAck);           -- [in std_logic]

  xfer_ack_2_opb_dff : FDR
    port map (
      Q => xfer_Ack_2_OPB,              -- [out std_logic]
      C => Clk,                         -- [in  std_logic]
      D => xfer_Ack_1_OPB,              -- [in  std_logic]
      R => Real_OPB_xferAck);           -- [in std_logic]

  xfer_ack_3_opb_dff : FDR
    port map (
      Q => xfer_Ack_3_OPB,              -- [out std_logic]
      C => Clk,                         -- [in  std_logic]
      D => xfer_Ack_2_OPB,              -- [in  std_logic]
      R => Real_OPB_xferAck);           -- [in std_logic]

  xfer_ack_4_opb_dff : FDR
    port map (
      Q => xfer_Ack_4_OPB,              -- [out std_logic]
      C => Clk,                         -- [in  std_logic]
      D => xfer_Ack_3_OPB,              -- [in  std_logic]
      R => Real_OPB_xferAck);           -- [in std_logic]

  xfer_Ack_OPB_i <= (xfer_Ack_1_OPB and not zbt_rw_I_n) or  -- Write transfer
                    xfer_Ack_4_OPB;                         -- Read transfer
  xfer_ack_opb_dff : FDR
    port map (
      Q => xfer_Ack_OPB,                                    -- [out std_logic]
      C => Clk,                                             -- [in  std_logic]
      D => xfer_Ack_OPB_i,                                  -- [in  std_logic]
      R => Real_OPB_xferAck);                               -- [in std_logic]

  xferAck_Video_DFF : process (Clk, Reset) is
  begin  -- process xferAck_DFF
    if Reset = '1' then                 -- asynchronous reset (active high)
      xfer_Ack_1_Video <= '0';
      xfer_Ack_2_Video <= '0';
      xfer_Ack_3_Video <= '0';
      xfer_Ack_4_Video <= '0';
    elsif Clk'event and Clk = '1' then  -- rising clock edge
      xfer_Ack_1_Video <= Video_CS;
      xfer_Ack_2_Video <= xfer_Ack_1_Video;
      xfer_Ack_3_Video <= xfer_Ack_2_Video;
      xfer_Ack_4_Video <= xfer_Ack_3_Video;
    end if;
  end process xferAck_Video_DFF;

  xfer_Ack_1 <= xfer_Ack_1_OPB or xfer_Ack_1_Video;
  xfer_Ack_3 <= xfer_Ack_3_OPB or xfer_Ack_3_Video;

  ZBT_xferAck   <= xfer_Ack_OPB;
  Video_xferAck <= xfer_Ack_4_Video;
end architecture IMP;
